/* ----------------------------------------------------------------------------
 *
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 *
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdm.h is required in all files using the RDM Database */
#include "rdm.h"

/* core04_db.h was generated from the DDL file core04_db.ddl by the ddlp.
 * utility.
 */
#include "core04_db.h"

/* core04_db_dbd.h was generated from the DDL file core04_db.ddl by the 
 * ddlp utility.  It is required to use the d_open_ptr API
 */
#include "core04_db_dbd.h"

/* EXAMPLE - Core for C */
/* Each time you run this example:
 *   1) The database is initialized (all existing contents removed)
 *   2) Several new artists are added to the database
 *   3) For each artist a couple albums are added to the database
 *   4) For each album a tracks are added to the database
 *   5) A list of each artist, their albums, and the album's tracks is 
 *      displayed
 *
 * The DDL (Database Definition Language) is in core04_db.ddl. The database
 * schema is embedded into the application and the database will be created 
 * the first time the application is run.
 *
 * The following RDM Functions are illustrated:
 *      d_opentask/d_closetask
 *      d_open_ptr/d_close
 *      d_initialize
 *      d_fillnew
 *      d_setor
 *      d_connect
 *      d_recfrst/d_recnext
 *      d_findfm/d_findnm
 *      d_recread
 *      d_errorname
 *      d_errorstr
 * They are fully documented in http://docs.raima.com/rdm/.
 *
 * Because the database is opened in exclusive mode lock and transactions
 * are not required.  Future core examples will use shared mode and will
 * demonstrate the use of transactions and locks.
 *
 * Each of these functions returns an integer status code, where the
 * value S_OKAY indicates a successful call.
 *
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDM call.
 *
 * The actual database will be stored in a directory named 'core04_db' in the
 * project directory.
 */

extern const char *the_doors_tracks[11];
extern const char *strange_days_tracks[10];
extern const char *waiting_tracks[11];
extern const char *the_rolling_stones_tracks[12];
extern const char *out_of_our_heads_tracks[12];
extern const char *beggars_banquet_tracks[10];
extern const char *tattoo_you_tracks[9];
extern const char *bleach_tracks[11];
extern const char *nevermind_tracks[13];
extern const char *in_utero_tracks[12];

/* This function assumes that the artist for the album is set as the current
   owner of the ARTIST_ALBUM set
 */
int32_t add_album(
    const char *album_title,
    uint16_t    num_tracks,
    const char *track_names[],
    DB_TASK    *task,
    int32_t     dbn)
{
    int32_t      ii;
    struct album album_rec;
    struct track track_rec;

    strcpy(album_rec.title, album_title);

    /* Add the new album record and associate it to the current artist */
    d_fillnew(ALBUM, &album_rec, task, dbn);
    d_connect(ARTIST_ALBUM, task, dbn);

    /* Make the new album owner of the ALBUM_TRACK set */
    d_setor(ALBUM_TRACK, task, dbn);

    /* Create new tracks and associate them to the album */
    for (ii = 0; ii < num_tracks; ii++)
    {
        strcpy(track_rec.name, track_names[ii]);

        d_fillnew(TRACK, &track_rec, task, dbn);
        d_connect(ALBUM_TRACK, task, dbn);
    }
    return S_OKAY;
}

/* =======================================================================
*/
int32_t main(void)
{
    DB_TASK       *task;
    int32_t        status;
    struct artist  artist_rec;
    struct album   album_rec;
    struct track   track_rec;

    /* create the context for the database operations */
    d_opentask(&task);

    /* open the database for exclusive access, this will create the database
     * dictionary (core04_db.dbd) file if it does not already exist.
     */
    status = d_open_ptr("core04_db", "x", &core04_db_dbd, sizeof(core04_db_dbd), task);
    if (status != S_OKAY)
    {
        fprintf (stderr, "Exclusive open failed, error = %s: %s\n", d_errorname(status), d_errorstr(status) );
        d_closetask (task);
        return (1);
    }

    /* Initializing the database will create the data files if they do not exist.
     * If the the data files already exist they will be removed and empty
     * files will be created.
     */
    d_initialize(task, CURR_DB);
    
    /* Create an artist record instance and add some of their albums */
    strcpy(artist_rec.name, "The Doors");
    d_fillnew(ARTIST, &artist_rec, task, CURR_DB);
    d_setor(ARTIST_ALBUM, task, CURR_DB);
    add_album("The Doors", RLEN(the_doors_tracks), the_doors_tracks, task, CURR_DB);
    add_album("Strange Days", RLEN(strange_days_tracks), strange_days_tracks, task, CURR_DB);
    add_album("Waiting for the Sun", RLEN(waiting_tracks), waiting_tracks, task, CURR_DB);

    
    /* Create an artist record instance and add some of their albums */
    strcpy(artist_rec.name, "The Rolling Stones");
    d_fillnew(ARTIST, &artist_rec, task, CURR_DB);
    d_setor(ARTIST_ALBUM, task, CURR_DB);
    add_album("The Rolling Stones", RLEN(the_rolling_stones_tracks), the_rolling_stones_tracks, task, CURR_DB);
    add_album("Out of our Heads", RLEN(out_of_our_heads_tracks), out_of_our_heads_tracks, task, CURR_DB);
    add_album("Beggars Banquest", RLEN(beggars_banquet_tracks), beggars_banquet_tracks, task, CURR_DB);
    add_album("Tattoo You", RLEN(tattoo_you_tracks), tattoo_you_tracks, task, CURR_DB);

    /* Create an artist record instance and add some of their albums */
    strcpy(artist_rec.name, "Nirvana");
    d_fillnew(ARTIST, &artist_rec, task, CURR_DB);
    d_setor(ARTIST_ALBUM, task, CURR_DB);
    add_album("Bleach", RLEN(bleach_tracks), bleach_tracks, task, CURR_DB);
    add_album("Nevermind", RLEN(nevermind_tracks), nevermind_tracks, task, CURR_DB);
    add_album("In Utero", RLEN(in_utero_tracks), in_utero_tracks, task, CURR_DB);

    /* Loop through each artist record and diplay the albums/tracks it
       is associated with
     */
    printf("\nList all artists and their albums/tracks\n");
    for (status = d_recfrst (ARTIST, task, CURR_DB);
         status == S_OKAY;
         status = d_recnext (task, CURR_DB))
    {
        /* Establish the current artist as the owner of the 
         * ARTIST_ALBUM set
         */
        d_setor(ARTIST_ALBUM, task, CURR_DB);

        /* Read and display the current artist record */
        d_recread(&artist_rec, task, CURR_DB);
        printf("\n%s\n", artist_rec.name);

        /* Find all of the album records related to this artist */
        for (status = d_findfm (ARTIST_ALBUM, task, CURR_DB);
             status == S_OKAY;
             status = d_findnm (ARTIST_ALBUM, task, CURR_DB))
        {
            /* Read and display the album information  */
            status = d_recread(&album_rec, task, CURR_DB);
            printf("\n\t%s\n", album_rec.title);

            /* Find all of the tracks related to this album */
            d_setor(ALBUM_TRACK, task, CURR_DB);
            for (status = d_findfm (ALBUM_TRACK, task, CURR_DB);
             status == S_OKAY;
             status = d_findnm (ALBUM_TRACK, task, CURR_DB))
            {
                /* Read and display the track information  */
                status = d_recread(&track_rec, task, CURR_DB);
                printf("\t\t%s\n", track_rec.name);
            }
        }
    }
    
    /* close the database */
    d_close (task);
    d_closetask (task);
    
    printf ("\nPress <Enter> to continue . . .\n");
    getchar ();
}

const char *the_doors_tracks[11] = 
{
    "Break on Through (To the Other Side)",
    "Soul Kitchen" ,
    "The Crystal Ship",
    "Twentieth Century Fox",
    "Alabama Song (Whisky Bar)",
    "Light My Fire",
    "Back Door Man",
    "I Looked at You",
    "End of the Night",
    "Take It as It Comes",
    "The End"
};

const char *strange_days_tracks[10] = 
{
    "Strange Days",
    "You're Lost Little Girl",
    "Love Me Two Times",
    "Unhappy Girl",
    "Horse Latitudes",
    "Moonlight Drive",
    "People Are Strange",
    "My Eyes Have Seen You",
    "I Can't See Your Face in My Mind",
    "When the Music's Over"
};

const char *waiting_tracks[11] = 
{
    "Hello, I Love You",
    "Love Street",
    "Not to Touch the Earth",
    "Summer's Almost Gone",
    "Wintertime Love",
    "The Unknown Soldier",
    "Spanish Caravan",
    "My Wild Love",
    "We Could Be So Good Together",
    "Yes, the River Knows",
    "Five to One"
};

const char *the_rolling_stones_tracks[12] =
{
    "Not Fade Away",
    "Route 66",
    "I Just Want to Make Love to You",
    "Honest I Do",
    "Now I've Got a Witness",
    "Little by Little",
    "I'm a King Bee",
    "Carol",
    "Tell Me",
    "Can I Get a Witness",
    "You Can Make It If You Try",
    "Walking the Dog"
};

const char *out_of_our_heads_tracks[12] =
{
    "She Said Yeah",
    "Mercy, Mercy",
    "Hitch Hike",
    "That's How Strong My Love Is",
    "Good Times",
    "Gotta Get Away",
    "Talkin' Bout You",
    "Cry to Me",
    "Oh, Baby (We Got a Good Thing Going)",
    "Heart of Stone",
    "The Under Assistant West Coast Promotion Man",
    "I'm Free"
};

const char *beggars_banquet_tracks[10] =
{
    "Sympathy for the Devil",
    "No Expectations",
    "Dear Doctor",
    "Parachute Woman",
    "Jig-Saw Puzzle",
    "Street Fighting Man",
    "Prodigal Son",
    "Stray Cat Blues",
    "Factory Girl",
    "Salt of the Earth"
};

const char *tattoo_you_tracks[9] =
{
    "Start Me Up",
    "Hang Fire",
    "Slave",
    "Little T&A",
    "Black Limousine",
    "Neighbours",
    "Worried About You",
    "Tops",
    "Heaven"
};

const char * bleach_tracks[11] =
{
    "Blew",
    "Floyd The Barber",
    "About a Girl",
    "School",
    "Love Buzz",
    "Paper Cuts",
    "Negative Creep",
    "Scoff",
    "Swap Meet",
    "Mr. Moustache",
    "Sifting"
};

const char * nevermind_tracks[13] =
{
    "Smells Like Teen Spirit",
    "In Bloom",
    "Come as You Are",
    "Breed",
    "Lithium",
    "Polly",
    "Territorial Pissings",
    "Drain You",
    "Lounge Act",
    "Stay Away",
    "On a Plain",
    "Something in the Way",
    "Endless, Nameless"
};

const char * in_utero_tracks[12] =
{
    "Serve the Servants",
    "Scentless Apprentice",
    "Heart-Shaped Box",
    "Rape Me",
    "Frances Farmer Will Have Her Revenge on Seattle",
    "Dumb",
    "Very Ape",
    "Milk It",
    "Pennyroyal Tea",
    "Radio Friendly Unit Shifter",
    "tourette's",
    "All Apologies"
};
